@@ -17,7 +17,7 @@ blavInspect <- function(blavobject, what, ...) {
     add.labels <- TRUE
     if(any(dotNames == "add.labels")) add.labels <- dotdotdot$add.labels
 
-    jagtarget <- inherits(blavobject@external$mcmcout, "runjags")
+    jagtarget <- lavInspect(blavobject, "options")$target == "jags"
   
     ## whats unique to blavaan
     blavwhats <- c("start", "starting.values", "inits", "psrf",
@@ -105,7 +105,6 @@ blavInspect <- function(blavobject, what, ...) {
             } else {
                 etas <- any(grepl("^eta", rownames(blavobject@external$stansumm)))
             }
-            if(!etas) stop("blavaan ERROR: factor scores not saved; set save.lvs=TRUE")
 
             ## how many lvs, excluding phantoms
             lvmn <- lavInspect(blavobject, "mean.lv")
@@ -114,9 +113,12 @@ blavInspect <- function(blavobject, what, ...) {
             }
             nlv <- length(lvmn)
 
+            if(nlv == 0) stop("blavaan ERROR: no latent variables are in the model")
+            if(!etas) stop("blavaan ERROR: factor scores not saved; set save.lvs=TRUE")
+            
             nsamp <- sum(lavInspect(blavobject, "nobs"))
 
-            draws <- make_mcmc(blavobject@external$mcmcout)
+            draws <- make_mcmc(blavobject@external$mcmcout, blavobject@external$stanlvs)
             drawcols <- grep("^eta", colnames(draws[[1]]))
 
             if(jagtarget){
